import abc
import json
from urllib import request

from many_repos import common
from many_repos import config


class BaseSource(abc.ABC):
    source_config: config.Source

    def __init__(self, source_config: config.Source):
        self.source_config = source_config

    # @abc.abstractmethod
    # def authenticate(self) -> bool:
    #     ...

    @abc.abstractmethod
    def get_repositories(self) -> list[common.Repository]:
        ...

    def filter_repositories(self, repositories: list[common.Repository]) -> list[common.Repository]:
        filtered_repositories = []
        for repo in repositories:
            if not self.source_config.forks and repo.fork:
                continue
            filtered_repositories.append(repo)
        return filtered_repositories

    @staticmethod
    def _make_request(
            url: str, method: str = "GET", *,
            body: dict | None = None, headers: dict[str, str] | None = None
    ) -> list[dict]:
        """
        Helper function to create http requests using urllib
        """
        if body:
            headers["Content-Type"] = "application/json"
            data = json.dumps(body).encode('utf-8')
        else:
            data = None

        req = request.Request(
            url=url,
            method=method,
            headers=headers,
            data=data
        )
        res = request.urlopen(req)
        return json.load(res)
