from many_repos import common
from many_repos.source_type.base import BaseSource


class Source(BaseSource):
    api_url = "https://api.github.com/user/repos?per_page=100?affiliation=owner"

    def get_repositories(self) -> list[common.Repository]:
        repos_json = self._make_request(self.api_url, headers=self._headers)
        repositories = []
        for repo in repos_json:
            if not self.source_config.forks and repo.get("fork"):
                continue
            namespace, name = repo.get("full_name").split("/", 2)
            repositories.append(
                common.Repository(
                    name=name,
                    namespace=namespace,
                    url=repo.get("ssh_url"),
                    fork=repo.get("fork"),
                    vcs="github.com"
                )
            )
        return repositories

    @property
    def _headers(self):
        return {
            "Accept": "application/vnd.github+json",
            "Authorization": f"Bearer {self.source_config.token}"
        }
