from many_repos import common
from many_repos.source_type.base import BaseSource


class Source(BaseSource):
    def get_repositories(self) -> list[common.Repository]:
        repos_json = self._make_request(self._api_url, headers=self._headers)
        repositories = []
        for repo in repos_json:
            namespace, name = repo.get("path_with_namespace").split("/")
            repositories.append(
                common.Repository(
                    name=name,
                    namespace=namespace,
                    url=repo.get("ssh_url_to_repo"),
                    fork=bool(repo.get("forked_from_project")),
                    vcs="gitlab.com"
                )
            )
        return repositories

    @property
    def _api_url(self) -> str:
        return f"https://gitlab.com/api/v4/users/{self.source_config.username}/projects"

    @property
    def _headers(self) -> dict[str, str]:
        return {
            "PRIVATE-TOKEN": self.source_config.token
        }
